#include "MusicPlayer.h"

/**
 * @brief MusicPlayer ctor
 * @param filename name of the music file to play
 * @details Intializes SDL_AUDIO and Mix_OpenAudio - be aware that I did
 *          not find a way to check for these being already initialized
 *          separately, don't use this if you have other parts of your
 *          program using the SDL_mixer (but it is compatible with SoundPlayer)
 */
MusicPlayer::MusicPlayer(std::string filename) : filename(filename) {
    /* Check if SDL_AUDIO is already running */
    if(SDL_WasInit(SDL_INIT_AUDIO) == 0) {
        /* Initialize SDL_AUDIO */
        if(SDL_Init(SDL_INIT_AUDIO) < 0) {
            std::cerr << "Failed to init SDL audio: " << SDL_GetError() << std::endl;
        }
        /* Initialize Mixer */
        if(Mix_OpenAudio(44100, MIX_DEFAULT_FORMAT, 2, 2048) < 0) {
            std::cerr << "Failed to init SDL mixer: " << Mix_GetError() << std::endl;
        }
    }
    
    /* Load music file */
    music = Mix_LoadMUS(filename.c_str());
    if(music == nullptr) {
        std::cerr << "Failed to load music \"" << filename << "\": " << Mix_GetError() << std::endl;
    }
}

/**
 * @brief Filename getter
 * @return name of the music file stored
 */
std::string MusicPlayer::get_filename() const {
    return filename;
}

/**
 * @brief Starts music playback
 * @param loops Number of times to play the music, -1 for infinite
 */
void MusicPlayer::play(int loops) {
    if(Mix_PausedMusic() == 1) {
        Mix_ResumeMusic();
    }
    else {
        Mix_PlayMusic(music, loops);
    }
}

/**
 * @brief Pauses music playback
 */
void MusicPlayer::pause() {
    Mix_PauseMusic();
}

/**
 * @brief Completely stops music playback
 */
void MusicPlayer::stop() {
    Mix_HaltMusic();
}

/**
 * @brief MusicPlayer dtor, frees contained music
 */
MusicPlayer::~MusicPlayer() {
    Mix_FreeMusic(music);
    // This would be nice to do, but can be only done once
    //Mix_CloseAudio();
}

/**
 * @brief Decreases volume by 16 units (8 total steps, range is 0-128)
 */
void MusicPlayer::volume_down() {
    int current_volume = Mix_VolumeMusic(-1);
    if(current_volume > 0) {
        Mix_VolumeMusic(current_volume-16<0 ? 0 : current_volume-16);
    }
}

/**
 * @brief Increases volume by 16 units (8 total steps, range is 0-128)
 */
void MusicPlayer::volume_up() {
    int current_volume = Mix_VolumeMusic(-1);
    Mix_VolumeMusic(current_volume+16);
}

/**
 * @brief Current volume getter
 * @return the current music volume
 */
unsigned MusicPlayer::volume_get() const {
    return Mix_VolumeMusic(-1);
}

/**
 * @brief Directly sets volume
 * @param volume the new volume to be set
 */
void MusicPlayer::volume_set(unsigned volume) {
    Mix_VolumeMusic(volume);
}
